rm(list=ls())

# packages utiles
library(reshape2)
library(maptools)

# une matrice de flux 
tab<- read.table("matmig_1961_1965.csv", header=TRUE, sep=";",
                 stringsAsFactors=FALSE, encoding="UTF-8")
mat<-as.matrix(tab[,-1])
row.names(mat)<-tab[,1]
mat

#tableau avec flux dans les deux directions
tabflow1<-melt(mat, as.is = TRUE)
names(tabflow1) <- c("i","j","Fij")
tabflow2<-melt(t(mat), as.is = TRUE)
names(tabflow2) <- c("i","j","Fji")

tabflow<-merge(tabflow1,tabflow2,by=c("i","j"),all.x=TRUE,all.y=TRUE)
str(tabflow)
head(tabflow)

#calcul et ajout au tableau des sommes marginales

Fi<-apply(mat,1,sum) #colonnes
tabi<-as.data.frame(cbind(names(Fi),Fi))
names(tabi)<-c("i","Fi")
tabflow<-merge(tabflow,tabi,id="i",all.x=TRUE)

Fj<-apply(mat,2,sum) #lignes
tabj<-as.data.frame(cbind(names(Fj),Fj))
names(tabj)<-c("j","Fj")
tabflow<-merge(tabflow,tabj,id="j",all.x=TRUE)

head(tabflow)
tail(tabflow)

#fonds de carte et tracé des limites
fdc <- readShapeSpatial("tch10.shp")
head(fdc@data)
plot(fdc)

#calcul des centroides et visualisation des codes
centres <- cbind(fdc@data[,"CODE"],as.data.frame(coordinates(fdc)))
colnames(centres) <- c("CODE","X","Y")
centres$CODE<-as.character(centres$CODE)
text(centres$X,centres$Y,centres$CODE,col="blue",cex=0.8)
head(centres)

#ajout des coordonnées au fichier des flux
names(centres)<-c("i","Xi","Yi")
tabflow <-merge(tabflow,centres,by="i", all.x=TRUE)

names(centres)<-c("j","Xj","Yj")
tabflow <-merge(tabflow,centres,by="j", all.x=TRUE)

#calcul des distances euclidiennes
tabflow$Dij<-round(sqrt((tabflow$Xi-tabflow$Xj)**2+(tabflow$Yi-tabflow$Yj)**2)/1000,1)
head(tabflow)

#importation du fichier d'attributs
tab<- read.table("tchpop_1965_1990.csv", header=TRUE, sep=";",
                 stringsAsFactors=FALSE, encoding="UTF-8")
head(tab)

#sélection d'attributs utiles
att<-tab[,c("CODE","NOM","REP","POP65")]
names(att)<-c("i","NOMi","REPi","POPi")
#ajout des attributs au fichier des flux

tabflow <-merge(tabflow, att, by="i", all.x=TRUE)

names(att)<-c("j","NOMj","REPj","POPj")
tabflow <-merge(tabflow, att, by="j", all.x=TRUE)

#attributs relationnels

tabflow$REPij <- tabflow$REPi==tabflow$REPj
tabflow$BORDERij <- tabflow$REPi!=tabflow$REPj
head(tabflow)

#interaction spatiale

#suppression flux inutiles et contrôle typage variable

tabflow <-tabflow[tabflow$i!=tabflow$j,]
str(tabflow)

#choix du modèle
#fonction glm family = gaussian, poisson

#gravitaire classique (sans contrainte)

#mod<-glm(Fij~log(POPi)+log(POPj),data=tabflow,family=poisson) #15%

#masse et distance

#mod<-glm(Fij~log(POPi)+log(POPj)+log(Dij),data=tabflow,family=poisson) #73%

#masse distance et qualitatif
mod<-glm(Fij~log(POPi)+log(POPj)+log(Dij)+REPij,data=tabflow,family=poisson)

#contrainte sur les origines 
#mod<-glm(Fij~i+log(POPj)+log(Dij),data=tabflow,family=poisson)

#contrainte sur les destinations
#mod<-glm(Fij~log(POPi)+j+log(Dij),data=tabflow,family=poisson)

#double contrainte sur les origines et les destinations
mod<-glm(Fij~i+j+log(Dij),data=tabflow,family=poisson)

#paramètres estimés
summary(mod)

#analysis de la variance
anova(mod, test="Chisq")
expl<- round(100*(1-(mod$deviance/mod$null.deviance)),2)
print(paste("Deviance explained :",expl,"%"))

#ajout des valeurs estimées et des résidus (bruts et relatifs)
tabflow$Fij_estim <- round(mod$fitted.values, 2)
tabflow$Fij_resraw <- round(tabflow$Fij-tabflow$Fij_estim,2)
tabflow$Fij_resrel <- round(tabflow$Fij/tabflow$Fij_estim,2)
sign <- tabflow$Fij_resraw/abs(tabflow$Fij_resraw)
tabflow$Fij_reschi2 <- round(sign*(tabflow$Fij-tabflow$Fij_estim)**2/tabflow$Fij_estim,2)

#visualisation des plus forts résidus
tabflow <-tabflow[order(tabflow$Fij_reschi2),]
head(tabflow)
tail(tabflow)

#cartographie des résidus

#choix du seuil
tabres$RESij<-tabres$Fij_resraw
summary(tabflow$Fij_reschi2)
sd(tabflow$Fij_reschi2)
seuil<-400

# (E.3) choix des param?tres de cartographie
minsize <- 1
coef <- 0.15
#par(mfrow=c(1,2))
#par(mfrow=c(2,1))

#sélection des résidus positifs

sel <- tabflow[tabflow$Fij_reschi2>seuil,]
sel$size <- sel$Fij_reschi2/seuil
sel$XXi<-sel$Xi+coef*(sel$Xj-sel$Xi)
sel$XXj<-sel$Xj-coef*(sel$Xj-sel$Xi)
sel$YYi<-sel$Yi+coef*(sel$Yj-sel$Yi)
sel$YYj<-sel$Yj-coef*(sel$Yj-sel$Yi)
plot(fdc, col="lightyellow",border="gray",lwd=1)
fdc2 <- readShapeSpatial("tch2.shp")
plot(fdc2, border="black", lwd=1.5,add=TRUE)
arrows(sel$XXi, sel$YYi, sel$XXj, sel$YYj, col="red", lwd=sel$size, length=0.1, code=2)
text(centres$Xj, centres$Yj,centres$j,col="black",cex=0.8)
title(main="Résidus positifs")

#résidus négatifs 
sel<-tabflow[-tabflow$Fij_reschi2>seuil,]

tabflow$Fij_reschi2 <- -tabflow$Fij_reschi2
sel <- tabflow[tabflow$Fij_reschi2>seuil,]

sel$size <- sel$Fij_reschi2/seuil
sel$XXi <- sel$Xi+coef*(sel$Xj-sel$Xi)
sel$XXj <- sel$Xj-coef*(sel$Xj-sel$Xi)
sel$YYi <- sel$Yi+coef*(sel$Yj-sel$Yi)
sel$YYj <- sel$Yj-coef*(sel$Yj-sel$Yi)
plot(fdc, col="lightyellow",border="gray",lwd=1)
plot(fdc2, border="black", lwd=1.5, add=TRUE)
arrows(sel$XXi, sel$YYi, sel$XXj, sel$YYj, col="blue", lwd=sel$size,length=0.1, code=2)
text(centres$Xj,centres$Yj,centres$j,col="black",cex=0.8)
title(main="Résidus négatifs")

