rm(list=ls())

#chargement des packages
library(reshape2)
library(maptools)
library(rCarto)
library(RColorBrewer)

#chargement d'une matrice de flux
tab <- read.table("matmig_1961_1965.csv", header=TRUE, sep=";",
                     stringsAsFactors=FALSE)
str(tab)

mat <- as.matrix(tab[,-1])
row.names(mat) <- tab[,1]
mat
class(mat)

#création des matrices Fij, Fji, Fsum, Fdif / tobler carto des volumes et des différences
Fij <- mat
Fij[1:3,1:3]
Fji <- t(mat) #transposée
Fji[1:3,1:3]

#somme des volumes échangés (coopération)
FSij <- Fij+Fji

#solde des échanges (compétition)
FDij <- Fij-Fji #compétition

rm(mat, tab, Fji, FSij, FDij)

#calcul des sommes marginales : arrivée, départ, volume, solde, asymétrie

O <- apply(Fij, 1, sum) # 1 lignes et là FUN optionnel
D <- apply(Fij, 2, sum) # 2 colonnes
V <- O + D
S <- D - O
A <- round(S/V, 2)
param <- data.frame(cbind(O,D,V,S,A))
param$CODE <- row.names(param)
param

#cartographie rapide avec rCarto
#import des gé©ométries
carte <- "tch10.shp"
fdc <- readShapeSpatial(carte) #à simplifier
head(fdc@data)                 #pour connaître id du shape

#départ (stock)
mapCircles(shpFile=carte,
           baseCol=colors()[444],
           txtCex=2,
           fixedNorm=TRUE,
           radiusMax=0.75,
           circleCol="red",
           lgdRnd=1,
           posLeg="bottomleft",
           title="Emigrants 1961-1965",
           legend="nb. de migrants",
           width=17,
           shpId="CODE",
           df=param,
           dfId="CODE",
           var="O")

#arrivées (stock)
mapCircles(shpFile=carte,
           baseCol=colors()[444],
           txtCex=2,
           fixedNorm=TRUE,
           radiusMax=0.75,
           circleCol="red",
           lgdRnd=1,
           posLeg="bottomleft",
           title="Immigrants 1961-1965",
           legend="nb. de migrants",
           width=17,
           shpId="CODE",
           df=param,
           dfId="CODE",
           var="D")

#volume migratoire (stock)
mapCircles(shpFile=carte,
           baseCol=colors()[444],
           txtCex=2,
           fixedNorm=TRUE,
           radiusMax=0.75,
           circleCol="red",
           lgdRnd=1,
           posLeg="bottomleft",
           title="Volume migratoire 1961-1965",
           legend="nb. de migrants",
           width=17,
           shpId="CODE",
           df=param,
           dfId="CODE",
           var="V")

#solde migratoire (stock+typo)
param$Solde_abs <- abs(param$S)
param$Solde_sig <- as.factor((1+param$S)/abs(1+param$S))
levels(param$Solde_sig) <- c("negatif","positif")
param

mapCirclesTypo(shpFile=carte, 
               shpId="CODE",
               df=param ,
               dfId="CODE",
               var="Solde_abs",
               var2="Solde_sig",
               baseCol=colors()[444],
               width=17,
               palCol="Set1",
               NACol="grey",
               title="Solde migratoire 1961-1965",
               legendCircles="Solde migratoire",
               legendTypo="Bilan")

#asymétrie migratoire (taux)
mapChoropleth(shpFile=carte, 
              shpId="CODE",
              df=param ,
              dfId="CODE",
              var="A",
              nclass=4,
              fixBrks=TRUE,
              listBrks=c(-1.00,-0.40,-0.20,-0.10,0,0.10,0.20,0.40,1.00),     
              width=17,
              lgdRnd=2,
              palCol="RdYlBu",
              NACol="grey",
              title="Attractivité migratoire 1961-1965",
              legend="Solde/Volume")

#volume et asymétrie migratoire (stock+taux)
mapCirclesChoro(shpFile=carte, 
                shpId="CODE",
                df=param ,
                dfId="CODE",
                var="V",
                var2="A",
                nclass=4,
                fixBrks=TRUE,
                listBrks=c(-1.00,-0.40,-0.20,-0.10,0,0.10,0.20,0.40,1.00),                  
                width=17,
                lgdRndDistr=2,
                palCol="RdYlBu",
                NACol="grey",
                baseCol="lightyellow",
                title="Volume et Attractivité migratoire 1961-1965",
                legendCircles="Volume",
                legendDistr="Asymétrie (solde/volume)")

rm(A,D,O,S,V,carte, param)

#transformation d'une matrice de flux en tableau de couples de lieux .

matflow <- Fij
tabflow <- melt(matflow) # crée facteurs
names(tabflow) = c("i","j","Fij")
head(tabflow)
str(tabflow)
class(tabflow)

#transformation d'un tableau de couples de lieux en matrice de flux
tempo <- melt(tabflow,
            id.vars=c("i","j"),
            measure.vars="Fij")
tempo2 <- dcast(tempo,i~j)
matflow2 <- as.matrix(tempo2[,-1])
rownames(matflow2) <- tempo2[,1]

matflow2[1:5,1:5]

rm(matflow, matflow2, tempo, tempo2)

#ajout de coordonnées et de distance - cartographie des flux

#choix de la matrice à enrichir (INPUT)
matflow <- Fij
  
#transformation en tableau avec flux dans les deux directions

tabflow1 <- melt(matflow, as.is = TRUE) #pour ne pas avoir transfo en facteurs as.is
names(tabflow1) <- c("i","j","Fij")
str(tabflow1)
tabflow2 <- melt(t(matflow), as.is = TRUE)
names(tabflow2) <- c("i","j","Fji")
str(tabflow2)

tabflow <- merge(tabflow1, tabflow2, by=c("i","j"), 
                 all.x=TRUE, all.y=TRUE)
head(tabflow)

rm(Fij, tabflow1, tabflow2)

#calcul et ajout au tableau des sommes marginales

Fi <- apply(matflow ,1, sum)
tabi <- as.data.frame(cbind(names(Fi), Fi))
names(tabi) <- c("i","Fi")
tabflow <- merge(tabflow, tabi, id="i", all.x=TRUE)
  
Fj <- apply(matflow, 2, sum)
tabj <- as.data.frame(cbind(names(Fj),Fj))
names(tabj) <- c("j","Fj")
tabflow <- merge(tabflow, tabj, id="j",all.x=TRUE)
  
head(tabflow)
tail(tabflow)

rm(tabi, tabj, Fi, Fj)

#import du fond de carte : ajout coordonnées et distance 

#chargement du fonds de carte et tracé des limites
dev.off()
head(fdc@data)
plot(fdc)

#calcul des centroides et visualisation des codes
centres <- cbind(fdc@data[,"CODE"], as.data.frame(coordinates(fdc)))
colnames(centres) <- c("CODE","X","Y")
text(centres$X, centres$Y, centres$CODE, col="blue", cex=0.8)
head(centres)

#ajout des coordonnées au fichier des flux
coordi <- centres
names(coordi) <- c("i","Xi","Yi")
tabflow <- merge(tabflow, coordi, by="i", all.x=TRUE)

names(coordi) <- c("j","Xj","Yj")
tabflow <- merge(tabflow, coordi, by="j", all.x=TRUE)

#calcul des distances euclidiennes
tabflow$Dij <- round(sqrt((tabflow$Xi-tabflow$Xj)**2
                          +(tabflow$Yi-tabflow$Yj)**2)/1000, 1)
head(tabflow)


#cartographie des volumes de flux (FVij)

#sélection des flux à représenter (demi-matrice)

tabflow$FVij <- tabflow$Fij + tabflow$Fji
sel <- tabflow[tabflow$i < tabflow$j, ] #prendre moitié matrice car symétrique

#choix du seuil

summary(sel$FVij)
seuil <- mean(sel$FVij)
sel <- sel[sel$FVij > seuil,]
head(sel)

#choix de l'épaisseur maximale des flux

maxsize <- 6
sel$size <- maxsize*sel$FVij/max(sel$FVij)

#retouche des coordonnées pour éloigner les flèches des centres

coeff <- 0.15
sel$XXi <- sel$Xi+coeff*(sel$Xj-sel$Xi)
sel$XXj <- sel$Xj-coeff*(sel$Xj-sel$Xi)
sel$YYi <- sel$Yi+coeff*(sel$Yj-sel$Yi)
sel$YYj <- sel$Yj-coeff*(sel$Yj-sel$Yi)
 
#cartographie d'une ou plusieurs couches de carte (facultatif)

dev.off()
fdc<-readShapeSpatial("tch10.shp")
plot(fdc, col="lightyellow", border="gray", lwd=1)

fdc2 <- readShapeSpatial("tch2.shp")
plot(fdc2, border="black", lwd=1.5, add=TRUE)

#cartographie des flux

segments(sel$XXi, sel$YYi, sel$XXj, sel$YYj, col="red", lwd=sel$size)

#ajout des codes (Facultatif)

text(centres$X,centres$Y,centres$CODE,col="blue",cex=0.8)

#ajout du titre et du sous-titre (facultatif)

title(main="Principaux échanges bilatéraux (Fij+Fji) ")
